@@ -880,7 +880,7 @@ class Index:
 			return False
 		return True
 
-	def _mutate_model(definition_name, base_model, versions, target_version):
+	def _mutate_model(base_model, mutations):
 		# Init Mutation
 		###############
 		mutated_model = {}
@@ -903,74 +903,61 @@ class Index:
 					mutated_model[k] = Schemas.Field(input_name=k)
 		# Mutate
 		########
-		# Mutate in-order until we reach the target version
-		found = False
-		for version in Index._versions:
-			if version not in versions:
-				continue
-			mutations = versions[version]['mutations']
-			for mutation in mutations:
-				if mutation.action == VersionMutationType.AddField:
-					field = Schemas.Field(**mutation.config)
-					field_name = field.input_name
-					mutated_model[field_name] = field
-				elif mutation.action == VersionMutationType.RemoveField:
-					field = Schemas.Field(**mutation.config)
-					del mutated_model[field.input_name]
-				elif mutation.action == VersionMutationType.MutateField:
-					old_field = mutated_model[mutation.config['field_name']]
-					config = {
-							k:mutation.config[k]
-							for k in mutation.config
-							if k != 'field_name'
-						}
-					new_field = Schemas.Field(
-							input_name=old_field.input_name
-						)
-					new_field.mutate(**config)
-					mutated_model[mutation.config['field_name']] = new_field
-				elif mutation.action == VersionMutationType.RenameField:
-					old_field_name = mutation.config['field_name']
-					old_field = mutated_model[old_field_name]
-					del mutated_model[old_field_name]
-					mutated_model[mutation.config['new_field_name']] = old_field
-			if version == target_version:
-				found = True
-				break
-		if found is not True:
-			raise Config.exception_factory.create_internal_error(
-					definition_name,
-					'Target version not found: {0}.'.format(target_version)
-				)
+		for mutation in mutations:
+			if mutation.action == VersionMutationType.AddField:
+				field = Schemas.Field(**mutation.config)
+				field_name = field.input_name
+				mutated_model[field_name] = field
+			elif mutation.action == VersionMutationType.RemoveField:
+				field = Schemas.Field(**mutation.config)
+				del mutated_model[field.input_name]
+			elif mutation.action == VersionMutationType.MutateField:
+				old_field = mutated_model[mutation.config['field_name']]
+				config = {
+						k:mutation.config[k]
+						for k in mutation.config
+						if k != 'field_name'
+					}
+				new_field = Schemas.Field(
+						input_name=old_field.input_name
+					)
+				new_field.mutate(**config)
+				mutated_model[mutation.config['field_name']] = new_field
+			elif mutation.action == VersionMutationType.RenameField:
+				old_field_name = mutation.config['field_name']
+				old_field = mutated_model[old_field_name]
+				del mutated_model[old_field_name]
+				mutated_model[mutation.config['new_field_name']] = old_field
 		return mutated_model
 
-	def _add_inherited_fields(inherited_definition_name, version, model):
-		parent_definition = Index.get(inherited_definition_name, version)
-		inherited_fields = parent_definition.list_fields()
-		for field_name in inherited_fields:
-			model[field_name] = inherited_fields[field_name]
-
-	def _extend_model(data_format, version):
-		model = {}
-		for inherited_definition_name in data_format.inherits:
-			Index._add_inherited_fields(inherited_definition_name, version, model)
-		for field_name in data_format.model:
-			field = data_format.model[field_name]
-			for action in data_format.field_init_actions:
+	def _execute_field_init(model, field_init_actions):
+		result = {}
+		for field_name in model:
+			field = model[field_name]
+			for action in field_init_actions:
 				field = action(field_name, field)
 				if field is None:
-					break
+					break  # omitted field
 			if field is not None:
-				model[field_name] = field
-		for action in data_format.field_init_actions:
-			action.validate_schema_model(model)
-		data_format.model = model
+				result[field_name] = field
+		return result
 
-	def _extend_mutations(mutations, data_format, version):
-		# TODO: Should traverse inheritance heirarchy, not just parent
-		for inherited_definition_name in data_format.inherits:
-			inherited_definition = Index.get(inherited_definition_name, version)
-			mutations.extend(inherited_definition.get_mutations())
+	def _extend_model(schema, mutations, version):
+		model = {}
+		inherited_mutations = list(mutations)
+		for inherited_definition_name in schema.inherits:
+			parent_definition = Index.get(inherited_definition_name, version)
+			inherited_fields = parent_definition.list_fields()
+			for field_name in inherited_fields:
+				model[field_name] = inherited_fields[field_name]
+			parent_mutations = parent_definition.get_mutations()
+			inherited_mutations.extend(parent_mutations)
+			Index._execute_field_init(model, parent_definition.get_data_format().field_init_actions)
+		for k in schema.model:
+			model[k] = schema.model[k]
+		model = Index._mutate_model(model, mutations)
+		model = Index._execute_field_init(model, schema.field_init_actions)
+		return model, inherited_mutations
 
 	def register_definition(
 				name,
@@ -992,78 +979,26 @@ class Index:
 			):
 		if Utils.Empty.isempty(base_version) is True:
 			base_version = 'base'
-		if data_type is DataType.Object:
-			Index._extend_model(data_format, base_version)
-		base_definition = Definition(
-				name=name,
-				aliases=aliases,
-				description=description,
-				data_type=data_type,
-				data_format=data_format,
-				default_value=default_value,
-				examples=examples,
-				nullable=nullable,
-				deprecated=deprecated,
-				internal=internal,
-				rules=rules,
-				base_version=base_version,
-				final_version=final_version,
-				version=base_version,
-				version_conversion=base_version_conversion,
-				mutations={},
-				meta=meta
-			)
-		name = base_definition.get_name()
-		if base_version not in Index._versions_idx:
-			raise Config.exception_factory.create_internal_error(
-					name,
-					'Base version not found: {0}.'.format(base_version)
-				)
 		definition_versions = {}
-		latest_def_version = base_definition
+		latest_def_version = None
 		if Utils.Empty.isempty(versions) is True:
 			versions = {}
+		mutated_data_format = data_format
 		initial_version_index = Index._versions_idx[base_version]
 		for version in Index._versions[initial_version_index:]:
 			if Utils.Empty.isempty(final_version) is False:
 				if Index.compare_versions(version, final_version) == 1:
 					break
-			if version not in versions:
-				# Index version w/no mutations
-				latest_def_version = Definition(
-						name=latest_def_version.name,
-						aliases=latest_def_version.aliases,
-						description=latest_def_version.description,
-						data_type=latest_def_version.data_type,
-						data_format=latest_def_version.data_format,
-						default_value=latest_def_version.default_value,
-						examples=latest_def_version.examples,
-						nullable=latest_def_version.nullable,
-						deprecated=latest_def_version.deprecated,
-						internal=latest_def_version.internal,
-						rules=latest_def_version.rules,
-						base_version=latest_def_version.base_version,
-						final_version=latest_def_version.final_version,
-						version=version,
-						version_conversion=latest_def_version.version_conversion,
-						mutations={},
-						meta=latest_def_version.meta
-					)
-				definition_versions[version] = latest_def_version
-				continue
-			if Utils.Empty.isempty(final_version) is False:
-				if Index.compare_versions(version, final_version) == 1:
-					break
-			mutated_data_format = latest_def_version.get_data_format().copy()
 			mutations = []
-			if data_type == DataType.Object:
-				Index._extend_model(mutated_data_format, version)
-				mutated_data_format.model = Index._mutate_model(name, mutated_data_format.model, versions, version)
-				Index._extend_mutations(mutations, mutated_data_format, version)
-			mutations.extend(versions[version]['mutations'])
 			version_conversion = None
-			if 'conversion' in versions[version]:
-				version_conversion = versions[version]['conversion']
+			if version in versions:
+				if 'mutations' in versions[version]:
+					mutations = versions[version]['mutations']
+				if 'conversion' in versions[version]:
+					version_conversion = versions[version]['conversion']
+			if data_type == DataType.Object:
+				mutated_data_format = mutated_data_format.copy()
+				mutated_data_format.model, mutations = Index._extend_model(mutated_data_format, mutations, version)
 			mutated_definition = Definition(
 					name=name,
 					aliases=aliases,
@@ -1080,7 +1015,7 @@ class Index:
 					final_version=final_version,
 					version=version,
 					version_conversion=version_conversion,
-					mutations = mutations,
+					mutations=mutations,
 					meta=meta
 				)
 			definition_versions[version] = mutated_definition
