@@ -1,4 +1,4 @@
-from cro_validate.enum import DataType
+from cro_validate.enum import DataType, ValidationMode
 import cro_validate.classes.definition_classes as Definitions
 from cro_validate.classes.configuration_classes import Config
 import cro_validate.classes.parameter_classes as Parameters
@@ -13,17 +13,22 @@ class Schema:
 	case_sensitive = True
 	load_defaults = False
 	dependency_resolver = None
+	inherits = []
+	field_init_actions = []
 
 	def __init__(
 				self,
 				model,
+				version=Utils.Empty,
 				model_name=Utils.Empty,
 				allow_unknown_fields=Utils.Empty,
 				display_name=Utils.Empty,
 				case_sensitive=Utils.Empty,
 				load_defaults=Utils.Empty,
 				dependency_resolver=Utils.Empty,
-				return_unknown_fields=Utils.Empty
+				return_unknown_fields=Utils.Empty,
+				inherits=Utils.Empty,
+				field_init_actions=Utils.Empty
 			):
 		def _set_member(name, value):
 			if Utils.Empty.isempty(value):
@@ -41,19 +46,26 @@ class Schema:
 		_set_member('load_defaults', load_defaults)
 		_set_member('dependency_resolver', dependency_resolver)
 		_set_member('return_unknown_fields', return_unknown_fields)
+		_set_member('inherits', inherits)
+		_set_member('field_init_actions', field_init_actions)
 		if self.model_name is None:
 			self.model_name = model.__class__.__name__
 
 	def copy(self):
+		model_copy = {}
+		for k in self.model:
+			model_copy[k] = self.model[k].copy()
 		result = Schema(
-				model=self.model,
+				model=model_copy,
 				model_name=self.model_name,
 				allow_unknown_fields=self.allow_unknown_fields,
 				display_name=self.display_name,
 				case_sensitive=self.case_sensitive,
 				load_defaults=self.load_defaults,
 				dependency_resolver=self.dependency_resolver,
-				return_unknown_fields=self.return_unknown_fields
+				return_unknown_fields=self.return_unknown_fields,
+				inherits=[k for k in self.inherits],
+				field_init_actions=[a for a in self.field_init_actions]
 			)
 		return result
 
@@ -128,8 +140,18 @@ class Field:
 		self._set_member('default_value', default_value)
 		if self.input_name is None:
 			raise Exceptions.InternalError('Field', 'Input name must be specified')
-		if self.output_name is None:
-			self.output_name = self.input_name
+
+	def copy(self):
+		f = Field(
+				input_name=self.input_name,
+				output_name=self.output_name,
+				definition_name=self.definition_name,
+				required=self.required,
+				ignored=self.ignored,
+				unvalidated=self.unvalidated,
+				default_value=self.default_value
+			)
+		return f
 
 	def mutate(
 				self,
@@ -137,13 +159,17 @@ class Field:
 				required=Utils.Empty,
 				ignored=Utils.Empty,
 				unvalidated=Utils.Empty,
-				default_value=Utils.Empty
+				default_value=Utils.Empty,
+				input_name=Utils.Empty,
+				output_name=Utils.Empty
 			):
 		self._set_member('definition_name', definition_name)
 		self._set_member('required', required)
 		self._set_member('ignored', ignored)
 		self._set_member('unvalidated', unvalidated)
 		self._set_member('default_value', default_value)
+		self._set_member('input_name', input_name)
+		self._set_member('output_name', output_name)
 
 	# Broken
 	def to_json_dict(self, field_name):
@@ -176,20 +202,27 @@ class Field:
 			result['default_value_type'] = default_value_type
 		return result
 
+	def __str__(self):
+		s = 'input_name={0}, output_name={1}, definition_name={2}, required={3}, ignored={4}, unvalidated={5}, default_value={6}'.format(
+					self.input_name,
+					self.output_name,
+					self.definition_name,
+					self.required,
+					self.ignored,
+					self.unvalidated,
+					self.default_value
+			)
+		return s
 
 class Validator:
-	def _validate(self, validated, fqn, field_name, definition, value, **rules_kw):
-		results = Parameters.Index.ensure(validated)
-		cb = Definitions.Index.validate_input
-		model_result = self.model_validator(None, fqn, field_name, definition, value, cb, **rules_kw)
-		results[field_name] = model_result
-
 	def __init__(self, name, model_validator):
 		self.name = name
 		self.model_validator = model_validator
 
-	def __call__(self, results, fqn, field_name, definition, value, **rules_kw):
-		return self._validate(results, fqn, field_name, definition, value, **rules_kw)
+	def __call__(self, results, fqn, field_name, definition, value, mode, **rules_kw):
+		model_result = self.model_validator(None, fqn, field_name, definition, value, **rules_kw)
+		results[field_name] = model_result
+		return results
 
 	def is_field_required(self, field_name):
 		if field_name in self.model_validator.required:
@@ -199,108 +232,77 @@ class Validator:
 	def get_field_definition_name(self, field_name):
 		return self.model_validator.get_field_definition_name(field_name)
 
-	def get_field_input_name(self, field_name):
-		return self.model_validator.get_field_input_name(field_name)
-
 	def get_field_input_display_name(self, field_name):
 		return self.model_validator.get_field_input_display_name(field_name)
 
 	def list_field_names(self):
-		field_names = set()
-		field_names.update(self.model_validator.required)
-		field_names.update(self.model_validator.optional)
-		return field_names
-
-	def list_definition_names(self):
-		field_names = set()
-		field_names.update(self.model_validator.required)
-		field_names.update(self.model_validator.optional)
-		definition_names = set()
-		for field_name in field_names:
-			if field_name in self.model_validator.definition_names:
-				definition_names.add(self.model_validator.definition_names[field_name])
-			else:
-				definition_names.add(field_name)
-		field_names = definition_names
-		object_names = set()
-		results = set(field_names)
-		for field_name in field_names:
-			definition_name = field_name
-			if definition_name in self.model_validator.definition_names:
-				definition_name = self.model_validator.definition_names[field_name]
-			field = Definitions.Index.get(definition_name)
-			if field.data_type == DataType.Object:
-				results.update(field.validator.list_definition_names())
-				object_names.add(field_name)
-			elif field.data_type == DataType.Array:
-				pass
-			else:
-				pass
-		for name in object_names:
-			results.remove(name)
-		return results
+		result = [k for k in self.model_validator.schema.model]
+		return result
 
 
 class ModelValidator:
-	def __init__(self, schema):
+	def __init__(self, schema, version, mode):
 		self.schema = schema
+		self.version = version
+		self.mode = mode
 		self.required = set()
 		self.optional = set()
 		self.ignored = set()
 		self.unvalidated = set()
 		self.definition_names = {}
-		self.input_field_names = {}
-		self.input_value_names = {}
+		self.input_field_name_idx = {} # field => input value
+		self.input_value_name_idx = {} # input value => field
 		self.default_values = {}
 		self.input_field_display_names = {}
-		self.output_names = {}
-		self.output_value_names = {}
+		self.output_field_name_idx = {} # field => output value
+		self.output_value_name_idx = {} # output value => field
 		self.dependency_order = []
 		self.load_schema()
 		self.name = schema.model_name
 		if schema.display_name is not None:
 			self.name = schema.display_name
 
-	def add_required(self, names):
+	def _add_required(self, names):
 		self.required.update(names)
 
-	def add_optional(self, names):
+	def _add_optional(self, names):
 		self.optional.update(names)
 
-	def add_ignored(self, names):
+	def _add_ignored(self, names):
 		self.ignored.update(names)
 
-	def add_unvalidated(self, names):
+	def _add_unvalidated(self, names):
 		self.unvalidated.update(names)
 
-	def add_definition_names(self, definition_names):
+	def _add_definition_names(self, definition_names):
 		self.definition_names.update(definition_names)
 
-	def add_output_names(self, output_names):
-		self.output_names.update(output_names)
-		self.output_value_names.update({k:output_names[k] for k in output_names})
+	def _add_output_field_name_idx(self, output_field_name_idx):
+		self.output_field_name_idx.update(output_field_name_idx)
+		self.output_value_name_idx.update({k:output_field_name_idx[k] for k in output_field_name_idx})
 
-	def add_input_names(self, input_names):
+	def _add_input_names(self, input_names):
 		normalized = input_names
 		if not self.schema.case_sensitive:
+			# k is schema field name, input_names[k] is input_name
 			normalized = {k.lower():input_names[k].lower() for k in input_names}
 		self.input_field_display_names.update(input_names)
-		self.input_field_names.update(normalized)
-		self.input_value_names.update({normalized[k]:k for k in normalized})
+		self.input_field_name_idx.update(normalized)
+		self.input_value_name_idx.update({normalized[k]:k for k in normalized})
 
-	def add_default_values(self, default_values):
+	def _add_default_values(self, default_values):
 		for field_name in default_values:
 			value = default_values[field_name]
 			if Utils.Empty.isempty(value):
 				continue
 			self.default_values[field_name] = value
 
-	def update_dependency_order(self):
+	def _update_dependency_order(self):
 		dependent_fields = set()
 		independent_fields = set()
 		for entry in self.definition_names:
 			definition_name = self.definition_names[entry]
-			definition = Definitions.Index.get(definition_name)
+			definition = Definitions.Index.get(definition_name, self.version)
 			if definition.is_one_of():
 				dependent_fields.add(entry)
 			else:
@@ -316,7 +318,7 @@ class ModelValidator:
 				unvalidated,
 				definition_names={},
 				input_names={},
-				output_names={},
+				output_field_name_idx={},
 				default_values={}
 			):
 		if self.schema.case_sensitive:
@@ -326,15 +328,15 @@ class ModelValidator:
 			unvalidated = {k.lower() for k in unvalidated}
 			ignored = {k.lower() for k in ignored}
 			default_values = {k.lower():default_values[k] for k in default_values}
-		self.add_required(required)
-		self.add_optional(optional)
-		self.add_ignored(ignored)
-		self.add_unvalidated(unvalidated)
-		self.add_definition_names(definition_names)
-		self.add_input_names(input_names)
-		self.add_output_names(output_names)
-		self.add_default_values(default_values)
-		self.update_dependency_order()
+		self._add_required(required)
+		self._add_optional(optional)
+		self._add_ignored(ignored)
+		self._add_unvalidated(unvalidated)
+		self._add_definition_names(definition_names)
+		self._add_input_names(input_names)
+		self._add_output_field_name_idx(output_field_name_idx)
+		self._add_default_values(default_values)
+		self._update_dependency_order()
 
 	def load_schema(self):
 		required = set()
@@ -343,7 +345,7 @@ class ModelValidator:
 		unvalidated = set()
 		definition_names = {}
 		input_names = {}
-		output_names = {}
+		output_field_name_idx = {}
 		default_values = {}
 		fields = {}
 		if isinstance(self.schema.model, dict):
@@ -364,7 +366,7 @@ class ModelValidator:
 			if field.definition_name is not None:
 				field_definition_name = field.definition_name
 			definition_names[name] = field_definition_name
-			field_definition = Definitions.Index.get(field_definition_name)
+			field_definition = Definitions.Index.get(field_definition_name, self.version)
 			if field.required:
 				required.add(name)
 			else:
@@ -374,7 +376,7 @@ class ModelValidator:
 			if field.unvalidated:
 				unvalidated.add(name)
 			if field.output_name:
-				output_names[name] = field.output_name
+				output_field_name_idx[name] = field.output_name
 			if field.input_name:
 				input_names[name] = field.input_name
 			if not Utils.Empty.isempty(field.default_value):
@@ -388,7 +390,7 @@ class ModelValidator:
 				unvalidated=unvalidated,
 				definition_names=definition_names,
 				input_names=input_names,
-				output_names=output_names,
+				output_field_name_idx=output_field_name_idx,
 				default_values=default_values
 			)
 
@@ -399,30 +401,38 @@ class ModelValidator:
 		return definition_name
 
 	def get_field_input_name(self, field_name):
-		if field_name in self.input_field_names:
-			return self.input_field_names[field_name]
+		if field_name in self.input_field_name_idx:
+			return self.input_field_name_idx[field_name]
 		return field_name
 
 	def get_field_input_display_name(self, field_name):
-		if field_name in self.input_field_names:
+		if field_name in self.input_field_name_idx:
 			return self.input_field_display_names[field_name]
 		return field_name
 
 	def get_field_output_name(self, field_name):
-		if field_name in self.output_names:
-			return self.output_names[field_name]
+		if field_name in self.output_field_name_idx:
+			return self.output_field_name_idx[field_name]
 		return field_name
 
-	def _list_field_input_names(self, vector):
+	def _list_field_names_for_input(self, vector):
+		# If mode is input, map field names from input.
+		# If mode is output, the field name is the input name.
 		result = []
 		if isinstance(vector, dict):
 			for k in vector:
-				if self.schema.case_sensitive is False:
-					k = k.lower()
-				if k in self.input_value_names:
-					result.append(self.input_value_names[k])
+				if self.mode is ValidationMode.Input:
+					if self.schema.case_sensitive is False:
+						k = k.lower()
+					if k in self.input_value_name_idx:
+						result.append(self.input_value_name_idx[k])
+					else:
+						result.append(k)
 				else:
-					result.append(k)
+					if self.schema.case_sensitive is False:
+						result.append(k.lower())
+					else:
+						result.append(k)
 		else:
 			for attribute_name in dir(vector):
 				try:
@@ -432,21 +442,30 @@ class ModelValidator:
 						continue
 					if callable(getattr(vector, attribute_name)):
 						continue
-					if self.schema.case_sensitive is False:
-						attribute_name = attribute_name.lower()
-					if attribute_name in self.input_value_names:
-						result.append(self.input_value_names[attribute_name])
+					if self.mode is ValidationMode.Input:
+						if self.schema.case_sensitive is False:
+							attribute_name = attribute_name.lower()
+						if attribute_name in self.input_value_name_idx:
+							result.append(self.input_value_name_idx[attribute_name])
+						else:
+							result.append(attribute_name)
 					else:
-						result.append(attribute_name)
-				except:
-					pass
+						if self.schema.case_sensitive is False:
+							result.append(attribute_name.lower())
+						else:
+							result.append(attribute_name)
+				except Exception as ex:
+					# TODO: This was 'pass', why?
+					raise Exceptions.InternalError(attribute_name, 'Field name listing failed.', exception=ex)
 		return result
 
 	def _get_input_value_name(self, fqn, field_name):
-		value_name = field_name
-		if value_name in self.input_field_names:
-			value_name = self.input_field_names[field_name]
-		return value_name
+		if self.mode is ValidationMode.Output:
+			return field_name
+		result = field_name
+		if result in self.input_field_name_idx:
+			result = self.input_field_name_idx[field_name]
+		return result
 
 	def _get_input_value(self, fqn, field_name, source):
 		value_name = self._get_input_value_name(fqn, field_name)
@@ -466,8 +485,9 @@ class ModelValidator:
 
 	def _set_output_value(self, field_name, target, value):
 		value_name = field_name
-		if field_name in self.input_field_names:
-			value_name = self.input_field_names[field_name]
+		if self.mode is ValidationMode.Input:
+			if field_name in self.input_field_name_idx:
+				value_name = self.input_field_name_idx[field_name]
 		if isinstance(target, dict):
 			target[value_name] = value
 		else:
@@ -478,7 +498,7 @@ class ModelValidator:
 
 	def _resolve_field_definition(self, fqn, field_name, dependency_values, **kw):
 		definition_name = self.get_field_definition_name(field_name)
-		definition = Definitions.Index.get(definition_name)
+		definition = Definitions.Index.get(definition_name, self.version)
 		if definition.is_one_of():
 			if self.schema.dependency_resolver is None:
 				raise Config.exception_factory.create_input_error(fqn, 'Schema field dependency resolver missing.')
@@ -487,7 +507,7 @@ class ModelValidator:
 				if definition_name not in definition.get_data_format():
 					msg = "Schema field cannot be '{0}' (must be one of: {1})".format(definition_name, definition.get_data_format())
 					raise Config.exception_factory.create_internal_error(fqn, msg)
-				definition = Definitions.Index.get(definition_name)
+				definition = Definitions.Index.get(definition_name, version=self.version)
 			except Exception as ex:
 				raise Config.exception_factory.create_input_error(fqn, 'Schema field resolution failed: {0}.'.format(ex), exception=ex)
 		return definition
@@ -502,19 +522,19 @@ class ModelValidator:
 			delattr(target, src_field_name)
 			setattr(target, target_field_name, tmp)
 
-	def _iterate(self, validated, fqn, values, callback, **rules_kw):
+	def __call__(self, results, fqn, field_name, definition, values, **rules_kw):
 		# Some objects could have dangling __dict__ entries (e.g. WebCore context)
 		# and other objects shouldn't be smashed, so the original ref must be
 		# returned.
 		# SOLUTION: Overwrite defaults but always return the original
 		#           ref where possible.
-		result = Parameters.Index.ensure(validated)
+		results = Parameters.Index.ensure(results)
 		normalized = values
 		if isinstance(values, dict):
 			if self.schema.case_sensitive is False:
 				values = {k.lower():values[k] for k in values}
 			normalized = Parameters.Index(values)
-		field_names = self._list_field_input_names(normalized)
+		field_names = self._list_field_names_for_input(normalized)
 		kw = Parameters.Index()
 		missing = set(self.required)
 		# Build kw
@@ -532,7 +552,7 @@ class ModelValidator:
 					raise Config.exception_factory.create_input_error(Config.definition_name_strategy.get_fqn(fqn, display_name), 'Unknown field.')
 				else:
 					if self.schema.return_unknown_fields is True:
-						result[field_name] = self._get_input_value(fqn, field_name, normalized)
+						results[field_name] = self._get_input_value(fqn, field_name, normalized)
 					continue
 			if field_name not in self.ignored:
 				kw[field_name] = self._get_input_value(fqn, field_name, normalized)
@@ -549,29 +569,30 @@ class ModelValidator:
 			# Unvalidated
 			#############
 			if field_name in self.unvalidated:
-				result[field_name] = self._get_input_value(fqn, field_name, normalized)
+				results[field_name] = self._get_input_value(fqn, field_name, normalized)
 				continue
 			# Default Value
 			###############
-			field_definition = self._resolve_field_definition(child_fqn, field_name, result, **rules_kw)
+			field_definition = self._resolve_field_definition(child_fqn, field_name, results, **rules_kw)
 			if field_name not in kw:
 				if field_definition.has_default_value() and field_name in self.required:
-					result[field_name] = field_definition.get_default_value(field_name)
+					results[field_name] = field_definition.get_default_value(field_name)
 				continue
 			# Normal Field
 			##############
-			definition_result = callback(
+			definition_result = Definitions.Index.validate(
 					validated=None,
 					field_fqn=child_fqn,
 					field_name=field_name,
 					definition_or_name=field_definition,
 					value=kw[field_name],
 					version=field_definition.version,
+					mode=self.mode,
 					**rules_kw)
 			for definition_result_entry in definition_result:
 				self._set_output_value(
 						definition_result_entry,
-						result,
+						results,
 						self._get_output_value(
 								definition_result_entry,
 								definition_result
@@ -579,19 +600,11 @@ class ModelValidator:
 					)
 		# Output Names
 		##############
-		result_keys = self._list_field_input_names(result)
-		for field_name in result_keys:
-			input_value_name = self._get_input_value_name(fqn, field_name)
-			if field_name in self.output_names:
-				output_name = self.output_value_names[field_name]
-				self._rename_output(input_value_name, output_name, result)
-		return result
-
-	def __call__(self, validated, fqn, field_name, definition, values, cb, **rules_kw):
-		return self._iterate(
-				validated,
-				fqn,
-				values,
-				cb,
-				**rules_kw
-			)
+		if self.mode is ValidationMode.Output:
+			result_keys = self._list_field_names_for_input(results)
+			for field_name in result_keys:
+				if field_name in self.output_field_name_idx:
+					output_name = self.output_field_name_idx[field_name]
+					if output_name is not None:
+						self._rename_output(field_name, output_name, results)
+		return results
