@@ -40,4 +40,11 @@ class Boundaries:
 		if max_len < 1:
 			return []
 		boundary = ''.join([c for i in range(max_len + 1)])
-		return [boundary]
\ No newline at end of file
+		return [boundary]
+
+
+class VersionMutation:
+	def __init__(self, action, config, description):
+		self.action = action
+		self.config = config
+		self.description = description
